% This script runs the generate_freq_longitudinal_estimator function that
% is used to design the modal damping estimator as an example of using frequency
% domain cost functions.
%
% The script has 7 sections seperated by cells:
% 1) Defines the frequency domain cost function that weights
%    the noisy OSEM sensors. *You should put your own cost function there*. 
% 2) Generates the quadruple pendulum models. The full order quad model is
%    generated so all fundamental cross-couplings are included. The reduced
%    order longitudinal model is also generated for calculating the modal
%    damping estimator.
% 3) The modal damping control filters are generated
% 4) Generates the estimator by calling generate_freq_longitudinal_estimator
% 5) Combines everything into full closed loop system
% 6) Generates the SISO filter form of the modal damping. This is simply to
%    plot what the modal damping looks like, the code in this cell is not 
%    used anywhere else.
% 7) The money plot, shows the noise performance of the modal damping
%    designed using the frequency domain cost function.
%
% Brett Shapiro
% 19 March 2013

clear
close all
clc

% directories called from this file
addpath('QuadModel_Production')
addpath('ModalDamp_Files')
addpath('Noise_Files')
addpath('MATLAB')

% plot parameters
linesize = 3;
textsize = 25;

%% 1) *** Put your favorite cost filter here ***   Generate the cost function filter for noisy top mass sensor in the modal damping loop.
% Must be in zpk format. generate_freq_longitudinal_estimator is only setup
% for zpk.

% Cost function filter components. The number of poles and zeros must be equal to ensure that the LQR R matrix meets the positive defintie requirement.
cost_gain_scale = 1e-7; % overall gain scaling
[ze pe ke] = ellip(2,1,60,2*pi*10,'high','s'); % high pass elliptical filter with 10 Hz corner
[z1 p1 k1] = zpkdata(bump(10,5,10));z1=z1{1};p1=p1{1}; % 10 Hz 10x bump with a quality factor of 5
zu1 = [ze;z1]; pu1 = [pe;p1]; % zeros and poles of cost function
ku1 = ke*k1*sqrt(cost_gain_scale);          % gain of cost function
sensor_cost_filter = zpk(zu1,pu1,ku1);

% plot sensor noise cost function    
freq = 0.1:0.001:100;
[magnitude phase] = bode(sensor_cost_filter,2*pi*freq);
magnitude = squeeze(magnitude); phase = squeeze(phase);
figure,maxfig    
    subplot(211)
        loglog(freq,magnitude,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize)
        title('Cost function filtering weighting the badness of sensor noise top mass longitudinal sensor noise')
        ylabel('magnitude (badness/noise)')        
    subplot(212)
        semilogx(freq,phase,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize)
        ylabel('Phase (degrees)')
        xlabel('Frequency (Hz)')
        IDplot


%% 2) Generate quad pendulum models

quad_mod.full.plant = ssmake4pv2eMB5f_fiber('quadopt_fiber'); % full order quadruple pendulum model
quad_mod.full.num_dof     = 24; % number of degrees of freedom in the full order model
quad_mod.full.num_sensor  = 6;  % number of top mass sensors in the full order model
quad_mod.full.num_seismic = 6;  % number of seismic inputs in the full order model
quad_mod.longitudinal.plant = model_cut(quad_mod.full.plant,'l'); % delete all pendulum degrees of freedom except longitudinal
quad_mod.longitudinal.num_dof     = 4; % number of degrees of freedom in the reduced order longitudinal quadruple pendulum
quad_mod.longitudinal.num_sensor  = 1; % number of top mass sensors in the reduced order longitudinal quadruple pendulum
quad_mod.longitudinal.num_seismic = 1; % number of seismic inputs in the reduced order longitudinal quadruple pendulum

%% 3) Generate modal damping control

% Modal decomposition
[phi modes] = ModalDecomp2(quad_mod.longitudinal.plant); % modal decomposition; phi is modal vectors, modes is the mode frequencies
modal2euler = phi'\eye(quad_mod.longitudinal.num_dof);   % modal basis to euler basis transformation matrix

% modal damping filters for 4 longitudinal modes
control = generate_modal_vel_damp(quad_mod.longitudinal);

% modal feedback gains. The gains are approximately equal to the closed loop damping ratio, i.e. gains =(approximately) damping ratio = 1/(2*quality_factor).
modal_feedback_gains = diag(1./(2*pi*modes(1:4)*9.0)); % set nominal modal ringdown to 1/e in 9 seconds. Imperfect state estimation will cause the ring time to be >= 9 seconds.

% Longitudinal damping actuator availability matrix. This limits the longitudinal damping drive to just the top mass.
damp_actuator_availability = [1 0 0 0;0 0 0 0;0 0 0 0; 0 0 0 0];

% overal modal damping
filter_modal2euler = damp_actuator_availability*modal2euler*modal_feedback_gains*control.filters;

%% 4) Generate estimator using frequency domain cost function on noisy sensor

% Modal estimator. estimator.ss has 4 modal outputs for the 4 longitudinal
% modes. It has 5 inputs, the first 4 are the control forces at each stage, in the euler coordinates. 
% Only first of these four is used, for the top mass. The 5th and final input is
% for the noisy top mass longitudinal sensor.
% signal.
estimator = generate_freq_longitudinal_estimator(quad_mod.longitudinal,sensor_cost_filter);


%% 5) Generate closed loop system with estimator and modal damping

% pitch damping loop, just to knock the pitch Qs out of the way. No pitch damping noise is considered here.
Dqp = zpk(-2*pi*[0;7.8],-2*pi*[10;20;20;15.9],5.0700e+06);  % set to match a simple LASTI pitch damping filter

% hook together the pendulum, estimator, and damping into a feedback loop
plant_est_cont = append(quad_mod.full.plant,estimator.ss,filter_modal2euler,Dqp);
connection_matrix = [   7 -29   % negative of modal damping output to top mass longitudinal force input
                       31 -29   % negative of modal damping output to first estimator input
                       35   1   % top mass output to last (5th) estimator input
                       36  25   % first estimator output to first conrol input
                       37  26   % second estimator output to second conrol input
                       38  27   % third estimator output to third conrol input
                       39  28   % fourth estimator output to fourth conrol input
                       40   5   % top mass pitch output to pitch damping input
                       11 -33]; % negative pitch damping filter output to top mass pitch torque input
input  = [1 7 35];    % longitudinal seismic input, top mass longitudinal force input, sensor noise input
output = [1 7 13 19]; %  longitudinal displacements of all four stages
closed_loop = connect(plant_est_cont,connection_matrix,input,output); % full closed loop system

% plot the top mass to top mass damped response
freq = 0.1:0.001:10;
[magnitude phase] = bode(closed_loop(1,2),2*pi*freq);
magnitude = squeeze(magnitude); phase = squeeze(phase);
figure,maxfig
    subplot(211)
        loglog(freq,magnitude,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize,'YLim',[1e-6 0.01])
        title('Damped top mass to top mass longitudinal transfer function')
        ylabel('Displacement (m/N)')        
    subplot(212)
        semilogx(freq,phase,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize,'YLim',[-200 45],'YTick',linspace(-180,45,6))
        ylabel('Phase (degrees)')
        xlabel('Frequency (Hz)')
        IDplot

% plot the impulse response time from a longitudinal seismic input to a longitudinal test mass response
time = 0:0.001:100;
test_mass_longitudinal_response = impulse(closed_loop(4,1),time);
figure,maxfig
    plot(time,test_mass_longitudinal_response,'LineWidth',linesize),grid on
    set(gca,'FontSize',textsize)
    title('Test Mass Response to Impulse at Suspension Point')
    ylabel('Amplitude (some arbitrary unit)')
    xlabel('Time (seconds)')    
    IDplot
    
    
%% 6) FYI: The equivalent longitudinal SISO damping filter

% hook together the estimator and modal damping filters
est_cont = append(estimator.ss,filter_modal2euler);
connection_matrix = [   1 -5  % negative top damp drive signal to mode 1 estimator input
                        6  1  % mode 1 estimator output to mode 1 damp filter input
                        7  2  % mode 2 estimator output to mode 2 damp filter input
                        8  3  % mode 3 estimator output to mode 3 damp filter input
                        9  4];% mode 4 estimator output to mode 4 damp filter input
 input  = 5; % sensor signal input
 output = 5; % top mass damp drive
 SISOmodal_longitudinal = connect(est_cont,connection_matrix,input,output); % modal damping in the form of a SISO damping filter
 freq = 0.1:0.01:100;
 [magnitude phase] = bode(SISOmodal_longitudinal,2*pi*freq);
 magnitude = squeeze(magnitude); phase = squeeze(phase);
 figure,maxfig
    subplot(211)
        loglog(freq,magnitude,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize)
        title('Longitudinal modal damping in SISO filter form')
        ylabel('Damping drive (N/m)')        
    subplot(212)
        semilogx(freq,phase,'LineWidth',linesize),grid on
        set(gca,'FontSize',textsize)
        ylabel('Phase (degrees)')
        xlabel('Frequency (Hz)')
        IDplot
        
% modal damping loop gain
modal_damping_loopgain = SISOmodal_longitudinal*quad_mod.full.plant(1,7);
% figure,maxfig
%     bode(modal_damping_loopgain),grid on
 
%% 7) Noise performance
 
load seismic_noise_Laurent % outdated BSC-ISI stage 2 requirement  -> loads struct seismic_noise
load sensor_noise % OSEM sensor noise, T0900496-v4                 -> loads struct sensor_noise
 
% seismic contribution to the cavity signal from an individual quad pendulum
seis_motion = abs(squeeze(freqresp(closed_loop(4,1),2*pi*seismic_noise.freq))).*seismic_noise.amplitude';

% top mass longitudinal sensor noise contribution to the cavity signal from an individual quad pendulum
noise_motion = abs(squeeze(freqresp(closed_loop(4,3),2*pi*sensor_noise.freq))).*sensor_noise.amplitude'/sqrt(2); % sqrt(2) comes from averaging 2 longitudinal sensors

% sum of seismic and sensor noise contributions to the cavity for an individual quad pendulum
total_motion = sqrt(seis_motion.^2 + noise_motion.^2);

% plot cavity contribution results
freq = seismic_noise.freq;
figure,maxfig
    loglog(freq,seismic_noise.amplitude,'k',freq,sensor_noise.amplitude/sqrt(2),'--k',freq,seis_motion,freq,noise_motion,freq,total_motion,'LineWidth',linesize),grid on
    set(gca,'FontSize',28,'YLim',[1e-22 1e-6],'YTick',logspace(-22,-6,17),'XLim',[0.1 20])
    title('Contributions to the Test Mass Displacement along the longitudinal DOF with')
    ylabel('Amplitude (m/\surdHz)')
    xlabel('Frequency (Hz)')
    legend('ISI requirement','OSEM sensor noise','test mass response to ISI','Test mass response to sensor noise','Total test mass response',3);