function control = generate_modal_vel_damp(quad_mod_longitudinal)
% This function compiles the modal damping feedback control. The input is
% the sys variable generated by generate_system. The output is a structure
% control, where the matrix field control.filters lists the modal damping
% filters for each mode on the diagonal.
%
% The damping filters are scaled so that the closed loop damping ratios are
% approximated the value multiplying each filter. For example, a damping
% ratio of 0.1 for mode 1 is obtained (approximately) by closing the mode 1
% loop with 0.1*control.filters(1,1).
%
% Brett Shapiro
% 19 Jan 2012

[phi modes] = ModalDecomp2(quad_mod_longitudinal.plant); % eigenvector matrix (unitless) and mode frequency vector (units of Hz)
M = quad_mod_longitudinal.plant.b(quad_mod_longitudinal.num_dof+1:end,1+quad_mod_longitudinal.num_sensor:end)\eye(quad_mod_longitudinal.num_dof); % Inertia matrix
Mm = phi'*M*phi; % Modal inertia matrix
Mm = diag(diag(Mm)); % Ensuring the matrix is purely diagonal as it should be. Non-zero offdiagonal elements of order ~1e-15 (machine precision, see eps) usually enter due to numerical errors.
modal2euler = phi'\eye(quad_mod_longitudinal.num_dof); % modal to euler coordinate transformation for actuation forces

% compiling the modal damping filters
for ii=1:length(modes)

    control_filter=generate_one_dof_vel_filter3(modes(ii)*1,80); % basic modal feedback filter
    control_filter = control_filter*bump(10,2,0.1);%*bump(10,1.78,1/sqrt(40))*bump(10,1.78,1/sqrt(40));% Applying some additional 10 Hz lowpass
    control_filter = control_filter/abs(freqresp(control_filter,2*pi*modes(ii))); % Scale the filter to the same gain as the ideal derivative (aka velocity) feedback filter at the modal frequency.    
    dampscale = 4*pi*modes(ii)*Mm(ii,ii)/(phi(1,ii)*modal2euler(1,ii)); % scale factor applied to the ideal derivative feedback filter so that the user chosen modal feedback gain is approximately equal to the closed loop damping ratio for that mode.
    control_filter = dampscale*control_filter; % Now, if this feedback filter is applied to mode(ii) as k*control_filter, k is approximately equal to the damping ratio of the mode. Note Q = 1/(2*zeta), where Q is the quality factor and zeta is the damping ratio, which is approximately equal to k.
    
    filter_matrix(ii,ii)=control_filter; %#ok<AGROW>
end
control.filters=filter_matrix;