function [quadModel,in,out,pend] = generate_QUAD_Model_Production(freq,buildType,varargin)
% [quadModel,in,out,pend] = generate_QUAD_Model_Production(freq,buildType,svnDir,plotFlag,isDamped,filters,useModal,modalDofs)
% 
% Generates a model structure quadModel which contains a state-space (.ss)
% and frequency response (.f) of the model generated by the following
% models and parameters: 
%     switch buildType
%         case 'wire'
%             quadModel.modelName = 'ssmake4pv2eMB4f_wire';
%             quadModel.paramsName = 'quadopt_wire';
%         case 'fiber'
%             quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
%             quadModel.paramsName = 'quadopt_fiber';
%         case 'erm'
%             quadModel.modelName = 'ssmake4pv2eMB4f_wire';
%             quadModel.paramsName = 'quadopt_erm';
%         case 'thincp'
%             quadModel.modelName = 'ssmake4pv2eMB4f_wire';
%             quadModel.paramsName = 'quadopt_thincp';
%         case {'wirerehang','MainChain_GlassOnWire'}
%               quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
%               quadModel.paramsName = 'quadopt_wirerehang';
%      end
% found in [svnDir 'QUAD/Common/MatlabTools/QuadModel_Production/']. 
%
% NOTE: The above cases are the five buildTypes that are part of a
% production quad. Other buildTypes may be defined below, but should be
% used with great caution, and clear understanding of how and why those
% parameters differ from production.
%
% The input string buildType indicates whether the build of the QUAD is of
% metal/steel wire type or fused silica/fiber type.
% 
% INPUTS:
%     freq     : [1xn double]       Desired frequency vector over which to
%                                   calculate the frequency response of the
%                                   model 
%     buildType: [1xm double]       Type of QUAD; 'fiber','wire','thincp', 'wirerehang', or 'erm'
%   ------------------------- ^ required ^ --------------------------------
%   ------------------------- v varargin v --------------------------------
%     svnDir   : [1xj char]         Defines the location of the local
%                                   computer's checkout of the Suspension
%                                   SVN Repository 
%                                   [default] = '/ligo/svncommon/SusSVN/sus/trunk/';                   
%     plotFlag : true or false      Defines whether to plot a few key
%                                   features of the frequency response model
%                                   [default] = false;
%     isDamped : true or false      Defines whether closed loop model is
%                                   defined as a part of quadModel
%                                   [default] = false;
%     dampFilters  : [1x6] struct       structure array defining non-default
%                                   damping control filters
%                                   [default] = LegacyFilts
%
% OUTPUTS:
%     quadModel = 
%             modelName: [1xk char]            name of the model used           
%            paramsName: [1xp char]            name of the the parameter set used
%                    ss: [24x30 ss]            "raw," undamped, state space model produced by the above model and param set
%                     f: [24x30xn double]   freqnuency repsonse of the undamped state space model, 
%                                              projected on to the input frequency vector 
%                    in: [1x1 struct]          structure defining indices of inputs (see below)
%                   out: [1x1 struct]          structure defining indices of outputs (see below)
%            pendParams: [1x1 struct]          Structure containing all parameters
%                                              used to define the model (n's, d's, I's, etc, etc)
%              v----------------------v (if isDamped is set to true)
%           dampFilters: [1x1 struct]          structure containing the filters used to calculate the close loop model
%              dampedin: [1x1 struct]          structure defining indices of inputs (see below)
%             dampedout: [1x1 struct]          structure defining indices of outputs (see below)
%              dampedss: [30x36 ss]            damped state space model, with SISO loops closed around the state space model with
%                                              either defualt or user input damping filters.
%               dampedf: [30x36xn double]
%                                  Note: Inputs for each stage of isolation
%                                  to the model are in force [N] or torque
%                                  [N.m], except the ground stage (gnd),
%                                  which are in displacement ([m] or
%                                  [rad]). Outputs for all stages are in
%                                  displacement ([m] or [rad])
% 
%     
%     in:        [1x1] struct
%     out:       [1x1] struct
%         Both in and out are structures of the following format:
%             in.(isoStage).(inputType).(dof)
%         where 
%             isoStage: isolation stage to which ins and outs are connected 
%                 = {'gnd';...   % At sus. point
%                    'top';...   % M0 or R0, top mass
%                    'uim';...   % "L1", upper intermediate mass
%                    'pum';...   % "L2", pen-ultimate mass
%                    'tst'};     % "L3", test mass
%             inputType: loop location (and there units)
%                 = {'disp';...  % displacement units ([m] or [rad]) 
%                    'drive';... % force units ([N] or [N.m])
%                    'sensNoise'};  % in has units of displacement and out has
%                                % units of force (used for quadState =
%                                % 'damped' only) 
%             dof: coordinates are with respect to the optical axis / beam line
%                 = {'L';...     % longitudinal
%                    'T';...     % transverse
%                    'V';...     % vertical
%                    'R';...     % roll
%                    'P';...     % pitch
%                    'Y'};       % yaw
%          Note that the size of the damped quadModel is larger than the
%          undamped quadModel. So, when damping is turned on, in and out
%          return the ins and outs of the damped model. However, BOTH
%          damped and undamped ins and outs are stored in the quadModel
%          struct, such that a comparison between the two models can me
%          made. (quadModel.in and quadModel.out are undamped indices and
%          quadModel.dampedin and quadModel.dampedout are damped indices
% 
%      pend:             Structure containing all parameters
%                        used to define the model (n's, d's, I's, etc, etc)

%% Parse user input
switch nargin
    case 2
        svnDir = '/ligo/svncommon/SusSVN/sus/trunk/';
        plotFlag = false;
        isDamped = false;
    case 3
        svnDir = varargin{1};
        plotFlag = false;
        isDamped = false;
    case 4
        svnDir = varargin{1};
        plotFlag = varargin{2};
        isDamped = false;
        filters = [];
        useModal = false;
        modalDofs = [];
    case 5
        svnDir = varargin{1};
        plotFlag = varargin{2};
        isDamped = varargin{3};
        useModal = false;
        
        disp('You''ve chosen to create closed loop model with the default filters ...')
        filterDir = [svnDir 'QUAD/Common/FilterDesign/'];
        quadModel.dampFilters.fileName = [filterDir 'dampingfilters_QUAD_20121121.mat'];
    case 6 
        svnDir = varargin{1};
        plotFlag = varargin{2};
        isDamped = varargin{3};
        quadModel.dampFilters.fileName = varargin{4};
        useModal = false;
    case 8
        svnDir = varargin{1};
        plotFlag = varargin{2};
        isDamped = varargin{3};
        dampFilters = varargin{4};
        useModal = varargin{5};
        modalDofs = varargin{6};
end

set(0,'DefaultAxesFontSize',18)
modelDir = [svnDir 'QUAD/Common/MatlabTools/QuadModel_Production/'];

rspStage = ['top';'uim';'pum';'tst'];
dof = ['L';'T';'V';'R';'P';'Y'];
inputType = {'disp';'force';'filter'};
dispDriveUnits = {'m/m';'m/m';'m/m';'rad/rad';'rad/rad';'rad/rad'};
forceDriveUnits = {'m/N';'m/N';'m/N';'rad/N.m';'rad/N.m';'rad/N.m'};

%% Add paths
addpath(modelDir)

%% Define the model and parameter set
disp(['Building a model for a ' buildType ' QUAD suspension ...'])
switch buildType
    case {'wire','MainChain_DummyMass'}
        quadModel.modelName = 'ssmake4pv2eMB4f_wire';
        quadModel.paramsName = 'quadopt_wire';
    case {'fiber','MainChain_MonoMass'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_fiber';
    case {'wirerehang','MainChain_GlassOnWire'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_wirerehang';
    case {'erm','ETMRxnChain_DummyMass'}
        quadModel.modelName = 'ssmake4pv2eMB4f_erm';
        quadModel.paramsName = 'quadopt_erm';
    case {'thincp','ITMRxnChain_DummyMass'}
        quadModel.modelName = 'ssmake4pv2eMB4f_thincp';
        quadModel.paramsName = 'quadopt_thincp';
    case {'fiber_d3d4_plusplus1mm'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_fiber_d3d4_plusplus1mm';
    case {'fiber_d3d4_minusminus1mm'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_fiber_d3d4_minusminus1mm';
    case {'fiber_d3d4_plusminus1mm'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_fiber_d3d4_plusminus1mm';
    case {'fiber_d3d4_minusplus1mm'}
        quadModel.modelName = 'ssmake4pv2eMB5f_fiber';
        quadModel.paramsName = 'quadopt_fiber_d3d4_minusplus1mm';
    case {'erm_20120831'}
        quadModel.modelName = 'ssmake4pv2eMB4f_erm';
        quadModel.paramsName = 'quadopt_erm_20120831';
    case {'thincp_20120831'}
        quadModel.modelName = 'ssmake4pv2eMB4f_thincp';
        quadModel.paramsName = 'quadopt_thincp_20120831';
    case {'wire_d3d4minusflex3'}
        quadModel.modelName = 'ssmake4pv2eMB4f_wire';
        quadModel.paramsName = 'quadopt_wire_d3d4minusflex3';
    case {'erm_d4physical'}
        quadModel.modelName = 'ssmake4pv2eMB4f_erm';
        quadModel.paramsName = 'quadopt_erm_d4physical';
    case {'thincp_d4physical'}
        quadModel.modelName = 'ssmake4pv2eMB4f_thincp';
        quadModel.paramsName = 'quadopt_thincp_d4physical';
end
disp(['     Using QUAD model:' quadModel.modelName])
disp(['     Using Params file: ' quadModel.paramsName])

%% Create the open loop model
eval(['[mbquad,pend] = ' quadModel.modelName '(''' quadModel.paramsName ''');']);
%*********************************************
quadModel.ss = mbquad;
quadModel.pendParams = pend;
disp('Calculating the frequency response of the open loop model ...')
tic
quadModel.f = freqresp(quadModel.ss, 2*pi*freq);
disp(['   Finished open loop in ',num2str(toc),' seconds.'])

%% Define undamped model indices
[in,out] = define_quadModel_insandouts('openloop');
quadModel.in = in;
quadModel.out = out;

%% Append to model, and close the loop
if isDamped
    disp('Closing the loop ...')
    
    % Define damped model indices
    [in,out] = define_quadModel_insandouts('damped');
    quadModel.dampedin = in;
    quadModel.dampedout = out;
    
    % Define digital calibration
    
    
    %% Load Damping Filters
    
    filters = load(quadModel.dampFilters.fileName);
    for iDOF = 1:6
        quadModel.dampFilters.(dof(iDOF)).c = filters.calibFilter.(dof(iDOF)).c;
    end
    
    if useModal
        for iDOF = modalDofs
            quadModel.dampFilters.(dof(iDOF)).ss      = modal_damp_to_euler_converter(dof{iDOF}(1),quadModel.ss);
            quadModel.dampFilters.(dof(iDOF)).c       = zpk(dampFilters(iDOF).ss);
            quadModel.dampFilters.(dof(iDOF)).isModal = true;
        end
    end
    
    % dampFilters.c is a SISO model, so it has 1 input and 1 output
    % Since it is first in the append, it takes over the first index of the
    % inputs and outputs of the 30 in x 24 out quadModel.ss.
    %
    
    filters.nDOFs = 6;
    filtersPlusQuad.ss = append(quadModel.dampFilters.L.c,...
                                quadModel.dampFilters.T.c,...
                                quadModel.dampFilters.V.c,...
                                quadModel.dampFilters.R.c,...
                                quadModel.dampFilters.P.c,...
                                quadModel.dampFilters.Y.c,...
                                quadModel.ss);
                            
    
    %%% Connection matrix defines what inputs are connected to what
    %%% outputs, via the indices in the in and out structs. Again because
    %%% we have a SISO loop, (I use the L loop as an example) the input to
    %%% the filter (with index in.top.sensNoise.L) is connected to the
    %%% displacement output of the open loop model (with index
    %%% out.top.disp.L). The drive input to the filter (with index
    %%% in.top.drive) is connected to the drive output of the filters (with
    %%% index out.top.sensNoise). This is shown diagramatically below.
    %%%
    %%%                                +---[m/N]---+
    %%%  -->(+)-- in.top.drive ------->| quadModel |------- out.top.disp ---->+-->
    %%%      |                         +-[rad/N.m]-+                          |
    %%%      |                                                                |
    %%%      |                          ~ |  +--[N/m]--+                      |
    %%%      +-- out.top.sensNoise --< -1 |--| filters |-- in.top.sensNoise --+
    %%%                                 ~ |  +[N.m/rad]+
    %%%
    %%% Note that the feedback -1 should already be included in the filters.
                            
    %                  Inputs        (are connected to)       Outputs
    cnxnMatrix = [in.top.sensNoise.L                       out.top.disp.L       ;... % LONG Loop
                  in.top.drive.L                        +1*out.top.sensNoise.L  ;... % -1 for feed back should be taken care of by filters 
        
                  in.top.sensNoise.T                       out.top.disp.T       ;... % TRANS Loop
                  in.top.drive.T                        +1*out.top.sensNoise.T  ;... % -1 for feed back should be taken care of by filters
                 
                  in.top.sensNoise.V                       out.top.disp.V       ;... % VERT Loop
                  in.top.drive.V                        +1*out.top.sensNoise.V  ;... % -1 for feed back should be taken care of by filters
                 
                  in.top.sensNoise.R                       out.top.disp.R       ;... % ROLL Loop
                  in.top.drive.R                        +1*out.top.sensNoise.R  ;... % -1 for feed back should be taken care of by filters
                 
                  in.top.sensNoise.P                       out.top.disp.P       ;... % PITCH Loop
                  in.top.drive.P                        +1*out.top.sensNoise.P  ;... % -1 for feed back should be taken care of by filters
                 
                  in.top.sensNoise.Y                       out.top.disp.Y       ;... % YAW Loop
                  in.top.drive.Y                        +1*out.top.sensNoise.Y  ];   % -1 for feed back should be taken care of by filters
    
    %%% Define an array of the total number of ins and outs in the combined
    %%% model, soley for the purpose of the connection process
    [nOuts,nIns]=size(filtersPlusQuad.ss);
    modelInds.outs = 1:nOuts;
    modelInds.ins  = 1:nIns;
    
    quadModel.dampedss = connect(filtersPlusQuad.ss,...
                                 cnxnMatrix,...
                                 modelInds.ins,...
                                 modelInds.outs);
    
    %%%
    disp('Calculating the frequency response of the closed loop model ...')
    tic
    quadModel.dampedf = freqresp(quadModel.dampedss,2*pi*freq);
    disp(['   Finished closed loop in ',num2str(toc),' seconds.'])
    
    if plotFlag
        
        for iDOF = 1:length(dof)
            figure
            ll = loglog(freq,abs([squeeze(quadModel.dampedf(out.top.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.dampedf(out.uim.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.dampedf(out.pum.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.dampedf(out.tst.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:))]));
            title([dof(iDOF) ' to ' dof(iDOF)])
            set(ll,'LineWidth',4)
            xlabel('Frequency (Hz)')
            ylabel(['Magnitude (' dispDriveUnits{iDOF} ')'])
            grid on
            xlim([freq(1) freq(end)]);set(gca,'XTick',10.^(-2:2))
            ylim([1e-10 1e2]);set(gca,'YTick',10.^(-10:2))
            leg = legend(['GND > top'],...
                         ['GND > uim'],...
                         ['GND > pum'],...
                         ['GND > tst']);
            set(leg,'FontSize',14)
        end
        
        for iRspStage = 1:length(rspStage)
            for iDOF = 1:length(dof)
                figure
                ll = loglog(freq,abs([squeeze(quadModel.dampedf(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.top.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.dampedf(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.uim.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.dampedf(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.pum.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.dampedf(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.tst.drive.(dof(iDOF)),:))]));
                title([dof(iDOF) ' to ' dof(iDOF)])
                set(ll,'LineWidth',4)
                xlabel('Frequency (Hz)')
                ylabel(['Magnitude (' forceDriveUnits{iDOF} ')'])
                grid on
                xlim([freq(1) freq(end)]);set(gca,'XTick',10.^(-2:2))
                ylim([1e-10 1e2]);set(gca,'YTick',10.^(-10:2))
                leg = legend(['TOP > ' rspStage(iRspStage,:)],...
                             ['UIM > ' rspStage(iRspStage,:)],...
                             ['PUM > ' rspStage(iRspStage,:)],...
                             ['TST > ' rspStage(iRspStage,:)]);
                set(leg,'FontSize',14)
            end
        end
    end
else
    if plotFlag
        rspStage = ['top';'uim';'pum';'tst'];
        dof = ['L';'T';'V';'R';'P';'Y'];
        inputType = {'disp';'force';'filter'};
        dispDriveUnits = {'m/m';'m/m';'m/m';'rad/rad';'rad/rad';'rad/rad'};
        forceDriveUnits = {'m/N';'m/N';'m/N';'rad/N.m';'rad/N.m';'rad/N.m'};
        
        for iDOF = 1:length(dof)
            figure
            ll = loglog(freq,abs([squeeze(quadModel.f(out.top.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.f(out.uim.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.f(out.pum.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:)),...
                                  squeeze(quadModel.f(out.tst.disp.(dof(iDOF)), in.gnd.disp.(dof(iDOF)),:))]));
            title([dof(iDOF) ' to ' dof(iDOF)])
            set(ll,'LineWidth',4)
            xlabel('Frequency (Hz)')
            ylabel(['Magnitude (' dispDriveUnits{iDOF} ')'])
            grid on
            xlim([freq(1) freq(end)]);set(gca,'XTick',10.^(-2:2))
            ylim([1e-10 1e2]);set(gca,'YTick',10.^(-10:2))
            leg = legend(['GND > top'],...
                ['GND > uim'],...
                ['GND > pum'],...
                ['GND > tst']);
            set(leg,'FontSize',14)
        end
        
        for iRspStage = 1:length(rspStage)
            for iDOF = 1:length(dof)
                figure
                ll = loglog(freq,abs([squeeze(quadModel.f(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.top.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.f(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.uim.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.f(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.pum.drive.(dof(iDOF)),:)),...
                                      squeeze(quadModel.f(out.(rspStage(iRspStage,:)).disp.(dof(iDOF)), in.tst.drive.(dof(iDOF)),:))]));
                title([dof(iDOF) ' to ' dof(iDOF)])
                set(ll,'LineWidth',4)
                xlabel('Frequency (Hz)')
                ylabel(['Magnitude (' forceDriveUnits{iDOF} ')'])
                grid on
                xlim([freq(1) freq(end)]);set(gca,'XTick',10.^(-2:2))
                ylim([1e-10 1e2]);set(gca,'YTick',10.^(-10:2))
                leg = legend(['TOP > ' rspStage(iRspStage,:)],...
                    ['UIM > ' rspStage(iRspStage,:)],...
                    ['PUM > ' rspStage(iRspStage,:)],...
                    ['TST > ' rspStage(iRspStage,:)]);
                set(leg,'FontSize',14)
            end
        end
    end
end
%%

