function [pend] = quadopt_wire
% global pend

% Corresponds to case 20060509controlsmainasbuiltsym of model mbquadlite2lateral
% For use with matrix elements ssmake4pv2eMB4.m/symbexport4lat.m of 5/9/06 or later.
% An approximation to the as-built controls prototype (10/11/05 build, main chain).
% Same as 20060331asbuiltsym but with measured masses from T040229-12 
% and blade spring constants recomputed directly from bounce mode frequencies. 
% All as-built numbers except off-diagonal MOI components are zeroed.

%**************************************************************************
%Updated to include Metal Quad NP values at LASTI.
%
%Locations of referenced documents with no DCC #:
% "As Designed Parameter Set 2" -
%   http://www.eng-external.rl.ac.uk/advligo/Reviews/FRR/Documents/As%20designed%20parameter%20set2.pdf
% "Useful data for Noise Prototype Quad assembly" -
%   http://www.eng-external.rl.ac.uk/advligo/documents/Useful%20data%20for%20Noise%20Prototype%20Quad%20assembly.pdf
%
%Brett Shapiro June 2008
%**************************************************************************
%Updated to correspond to main chain wire hang configuration
% Still needs updated moments of inertia for round masses.
%
%Brett Shapiro Feb 2009
%**************************************************************************
% Electronics egain = 165.5769963469529 
% This is the measured gain of the
% LASTI electronics (including OSEMs). It was obtained by comparing the fit
% z model with the measured z top mass response at 10 Hz, where the only important mechanical
% paramter should be the top mass mass mn. mn was measure on a scale to +-
% 10 grams (better than that originally, but probably some drift from the work at
% LASTI)
%
% Brett Shapiro 29 March 2009
%**************************************************************************
% Updated to included parameters from the hierarchal model fit. This
% hierchal fit method fit the top mass response first, when it was free and the rest
% of the main chain was locked down. Then the UI mass was released and it's
% parameters were fit, assuming the top mass parameters were obtained from the first fit,
% then the penultimate was released...and so on.
%
% Additionally, the wire lengths were adjusted to reflect the noise
% prototype wire hang configuration. The lengths were sent in a spread
% sheet from RAL that included the wire jig tuning frequencies. All the
% previous values were slightly off and may have been a carry over from a
% previous quad design.
%
% The yaw moments of inertia (Iz) still need to be refined with the hierarchal
% fit. They also reflect the pre-PUM magnet state.
%
% Brett Shapiro 12 May 2009

% 17 May 2011
% commented out global pend at top of this file and quadopt...
% since no function calls are made global variables are not needed. Matlab
% also seems to have a bug where global variables are not completely
% cleared even though they do not show up in the command line workspace.
% The effect is that a parameter is updated or even removed entirely, the 
% old value might be carried through anyway.


                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%% Universal Parameters %%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            pend.version = '20060509noisemainasbuiltoffdiag_wire';
                            pend.g = 9.81;
                            pend.egain = 165.5769963469529; % Measured LASTI top mass electronics gain, BNS 3/29/09 (calibrated using z model with measured masses and blade stiffnesses)
                            pend.bd = 0.001; % internal viscous damping - BNS 6/24/2008 (somewhat arbitrary, 0 damping is inconvenient in matlab and also unrealistic)
                            pend.stage2 = 1;  
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%%%%% Top Mass %%%%%%%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % length, width, and hieght dimensions
                            pend.nx = 0.1300; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09
                            pend.ny = 0.5000; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09 
                            pend.nz = 0.0840; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09 
                            
                            % density
                            pend.denn = 4000; % T040214-01, also "As Designed Parameter Set 2"
                            
                            % inertias
                            pend.mn = 21.9294;  % T1000286-v5
                            pend.Inx = 0.46003463; % T1000286-v5
                            pend.Iny = 0.072723418; % T1000286-v5
                            pend.Inz = 0.47217665; % T1000286-v5
                            pend.Inxy = -0.037559948; % T1000286-v5
                            pend.Inyz = 0.000046546253; % T1000286-v5
                            pend.Inzx = 0.0017184099;   % T1000286-v5
                            
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%%%%% UI Mass %%%%%%%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % length, width, and hieght dimensions
                            pend.ux = 0.1300; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09
                            pend.uy = 0.5000; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09
                            pend.uz = 0.0840; % T040214-01, also "As Designed Parameter Set 2", BNS 5/15/09
                            
                            % density
                            pend.den1 = 4000; % T040214-01, also "As Designed Parameter Set 2"
                            
                            % inertias
                            pend.m1 = 22.338257; % T1000286-v5
                            pend.I1x = 0.50875622; % T1000286-v5
                            pend.I1y = 0.071054293; % T1000286-v5               
                            pend.I1z = 0.51808050; % T1000286-v5
                            pend.I1xy = -0.013211675; % T1000286-v5
                            pend.I1yz = 0; % T1000286-v5
                            pend.I1zx = 0; % T1000286-v5
%                             pend.I1x = 0.509; % "As Designed Parameter Set 2"; BNS 6/18/08
%                             pend.I1x = 0.515727291465044; % Model Fit: gradient_descent_fit_y_pitch_roll_PUM_locked; BNS 5/9/09                            
%                             pend.I1y = 0.0714; % "As Designed Parameter Set 2"; BNS 6/18/08
%                             pend.I1y = 0.070579025875477; % Model Fit: gradient_descent_fit_y_pitch_roll_PUM_locked; BNS 5/9/09                            
%                             pend.I1z = 0.518; % "As Designed Parameter Set 2"; BNS 6/18/08
                            
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%%%%% PUM Mass %%%%%%%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % width and radius
                            pend.ix = 0.200; % "As Designed Parameter Set 2", BNS 5/15/09
                            pend.ir = 0.170; % "As Designed Parameter Set 2", BNS 5/15/09
                            
                            % density
                            pend.den2 = 3860; % T040214-01
                            
                            % inertias
                            pend.m2 = 39.63 + 0.761; % BNS -  measured 1/20/09, +0.761 adjusted April 2009 for PUM magnet installation
                            pend.I2x = 0.680066498028610; % Model Fit: gradient_descent_fit_y_pitch_roll_Test_locked; BNS 5/15/09
                            pend.I2y = 0.439440905374475;  % Model Fit: gradient_descent_fit_y_pitch_roll_Test_locked; BNS 5/15/09
                            pend.I2z = 0.41557; %Model Fit: gradient_descent_fit_yaw.m; BNS 7/29/2008
                            pend.I2xy = 0;
                            pend.I2yz = 0;
                            pend.I2zx = 0;
%                             pend.I2x = 0.608142; % D060355-D; BNS 6/18/08 
%                             pend.I2x = 0.658291447460908; % Model Fit: gradient_descent_fit_y_pitch_roll_Test_locked; BNS 5/9/09
%                             pend.I2y = 0.404138; % D060355-D; BNS 6/18/08 
%                             pend.I2y = 0.437792361035363;  % Model Fit: gradient_descent_fit_y_pitch_roll_Test_locked; BNS 5/9/09
%                             pend.I2z = 0.397621; % D060355-D; BNS 6/18/08
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%%%%% Test Mass %%%%%%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % width and radius
                            pend.tx = 0.200; % 'Proposed Steel Prism Position (from D060039)', BNS 5/15/09, also from "As Designed Parameter Set 2"
                            pend.tr = 0.170; % 'Proposed Steel Prism Position (from D060039)', BNS 5/15/09, also from "As Designed Parameter Set 2"
                            
                            % density
                            pend.den3 = 2201; % density of fused silica (see note below). BNS 5/15/09 ("As Designed Parameter Set 2" says 2200)
                            
                            % inertias
                            pend.m3 = 39.6; % BNS -  measured 1/20/09
                            pend.I3x = 0.597752517013395; % Model Fit: gradient_descent_fit_y_pitch_roll; BNS 5/9/09
                            pend.I3y = 0.418422904155540; % Model Fit: gradient_descent_fit_y_pitch_roll; BNS 5/9/09
                            pend.I3z = 0.40046; %Model Fit: gradient_descent_fit_yaw.m; BNS 7/29/2008
                            pend.I3xy = 0;
                            pend.I3yz = 0;
                            pend.I3zx = 0; 
%                             pend.I3x = 0.606789; % D060355-D; BNS 6/18/08
%                             pend.I3y = 0.409628; % D060355-D; BNS 6/18/08 
%                             pend.I3z = 0.396568; % D060355-D; BNS 6/18/08
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %%%%%%%%%% Wires %%%%%%%%%%%
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % Lengths
                            pend.ln = 0.445; % RAL spreadsheet. Updated by BNS 8 May 2009
                            pend.l1 = 0.311; % RAL spreadsheet. Updated by BNS 8 May 2009
                            pend.l2 = 0.339; % RAL spreadsheet. Updated by BNS 8 May 2009. Should've actually been 0.328; leaving as is for historical purposes; BNS 27 July 2010.
%                             pend.l3 = 0.604; % RAL spreadsheet. Updated by BNS 8 May 2009
                            pend.l3 = 0.604027847040184; % calculated for updated wire hang pend.n5, pend.l3 = sqrt(0.604 + (pend.n5-pend.n4)^2), BNS 5/9/09
%                             pend.ln = 0.448574; 
%                             pend.l1 = 0.308993; % outdated values
%                             pend.l2 = 0.341653; 
%                             pend.l3 = 0.600648; 
                            
                            % Radii
                            pend.rn = (1.1/2)*10^-03; % "Useful data for Noise Prototype Quad assembly"; BNS 6/18/08
                            pend.r1 = (0.71/2)*10^-03; % "Useful data for Noise Prototype Quad assembly"; BNS 6/18/08
                            pend.r2 = (0.635/2)*10^-03; % "Useful data for Noise Prototype Quad assembly"; BNS 6/18/08
                            pend.r3 = (0.457/2)*10^-03; % "Useful data for Noise Prototype Quad assembly"; BNS 6/18/08
%                             pend.r3 = (0.4/2)*10^-3; % For Silica Fibers. Alastair H 11/11/2008
                            
                            % Number at each stage
                            pend.nwn = 2;
                            pend.nw1 = 4;
                            pend.nw2 = 4;
                            pend.nw3 = 4;
                            
                            % Moduli of elasticity
                            pend.Yn = 2.12000*10^+11; % "As Designed Parameter Set 2"; BNS 6/18/08
                            pend.Y1 = 2.12000*10^+11; % "As Designed Parameter Set 2"; BNS 6/18/08
                            pend.Y2 = 2.12000*10^+11; % "As Designed Parameter Set 2"; BNS 6/18/08
                            pend.Y3 = 2.12000*10^+11; % BNS 6/18/08
%                             pend.Y3 = 7.2*10^10; % For Silica Fibers. Peter Fritschel 11/11/2008

                            % Obsolete: ribbon parameters
                            pend.twistlength = 0; 				% T040214-01 
                            pend.d3tr = 1.0000*10^-03;	% T040214-01 
                            pend.d4tr = 1.0000*10^-03;	% T040214-01 
                            
                            % X distance of wire from center line
                            pend.sn = 0; 	  % T040214-01 
                            pend.su = 0.003;  % T040214-01 
                            pend.si = 0.003;  % T040214-01 
                            pend.sl = 0.015;  % T040214-01 
                            
                            % y distance of wire attatchment points from center
                            pend.nn0 = 0.250;  % T040214-01, also "As Designed Parameter Set 2"
                            pend.nn1 = 0.090;  % T040214-01, also "As Designed Parameter Set 2"
                            pend.n0 = 0.200;  % T040214-01, also "As Designed Parameter Set 2" 
                            pend.n1 = 0.060;  % T040214-01, also "As Designed Parameter Set 2" 
                            pend.n2 = 0.140;  % T040214-01, also "As Designed Parameter Set 2" 
                            pend.n3 = 0.1762; % CT, email to NR, 9/22/04, also "As Designed Parameter Set 2" 
                            pend.n4 = 0.1712; % CT, email to NR, 9/22/04, also "As Designed Parameter Set 2"
                            pend.n5 = 0.1712; % CT, email to NR, 9/22/04, also "As Designed Parameter Set 2"                      
%                             pend.n3 = 0.17095; % BNS 5/9/09, determined from D060358 and D060338
%                             pend.n4 = 0.17095; % BNS 5/9/09, determined from D060358 and D060338 
                            
                            % sine of angle of wires
                            sin = (pend.nn1-pend.nn0)/pend.ln;
                            si1 = (pend.n1-pend.n0)/pend.l1;
                            si2 = (pend.n3-pend.n2)/pend.l2;
                            si3 = (pend.n5-pend.n4)/pend.l3;                           
                            
                             
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            % Blade Spring Stiffnesses %
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % Z Stiffnesses                                
                                
							% Solved for exactly using the measured masses and resonance frequencies. 
                            % See matlab code spring_stiffness_calc.m: BNS 6/19/08.
							% Note that these stiffnesses represent individual springs; multiply by 2 to get the siffness between the masses.
                            pend.kcn = 1411.4640262910420948351545480224;
							pend.kc1 = 1650.5240590453796728792143217792;
							pend.kc2 = 2423.5190152802382310519860608100;
%                             pend.kcn = 1611.1; % "As Designed Parameter Set 2" BNS 6/18/08 
% 							  pend.kc1 = 1811.4; % "As Designed Parameter Set 2" BNS 6/18/08 
% 							  pend.kc2 = 2630.3; % "As Designed Parameter Set 2" BNS 6/18/08  
                            
                            % X stiffnesses
                            pend.kxn = 1.0*10^5; % as for middle, also "As Designed Parameter Set 2"
                            pend.kx1 = 1.0*10^5; % Justin 11/29/05, also "As Designed Parameter Set 2"
                            pend.kx2 = 0.8*10^5; % "As Designed Parameter Set 2"            
                            
                            % Blade stiffness correction parameters. Mark
                            % Barton said that the precise values here don't
                            % matter since the z stiffnesses above were already fit. BNS 5/27/09 
                            ffn = 0.807; % from Ian's data, 11/30/05 
                            ff1 = 0.641; % from Ian's data, 11/30/05 
                            ff2 = 0.608; % from Ian's data, 11/30/05                             
                            kffn = 1 + ffn*tan(asin(sin));
                            kff1 = 1 + ff1*tan(asin(si1));
                            kff2 = 1 + ff2*tan(asin(si2));
                            
                            
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            % d's (wire z break offs)  %
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            
                            % Added to pend structure definition for
                            % convenience of d's. T080096-00-K. BNS 6/18/2008
                            pend.mn3 = pend.mn+pend.m1+pend.m2+pend.m3;
                            pend.m13 = pend.m1+pend.m2+pend.m3;
                            pend.m23 = pend.m2+pend.m3;
                            pend.cn=(pend.ln^2-(pend.nn1-pend.nn0)^2)^0.5/pend.ln;	%cos(omegan)
                            pend.c1=(pend.l1^2-(pend.n1-pend.n0)^2)^0.5/pend.l1;	%cos(omega1)
                            pend.c2=(pend.l2^2-(pend.n3-pend.n2)^2)^0.5/pend.l2;	%cos(omega2)
                            pend.c3=(pend.l3^2-(pend.n5-pend.n4)^2)^0.5/pend.l3;	%cos(omega3)
                            pend.flexn = sqrt(2*((1/4)*pi*pend.rn^4)*pend.Yn/pend.mn3/pend.g)*pend.cn^(3/2); % flexn = 0.004534992483623; 
                            pend.flex1 = sqrt(4*((1/4)*pi*pend.r1^4)*pend.Y1/pend.m13/pend.g)*pend.c1^(3/2); % flex1 = 0.002747104240021; 
                            pend.flex2 = sqrt(4*((1/4)*pi*pend.r2^4)*pend.Y2/pend.m23/pend.g)*pend.c2^(3/2); % flex2 = 0.002926142794287; 
                            pend.flex3 = sqrt(4*((1/4)*pi*pend.r3^4)*pend.Y3/pend.m3/pend.g)*pend.c3^(3/2); % flex3 = 0.002163035309137; for long/pitch, yaw and vertical
                                                                           
                            %d correction for lateral blade compliance.
                            %Added to pend structure definition for
                            %convenience of d's. T080096-00-K. BNS 6/18/2008
                            pend.bend1 = pend.g*(pend.m1+pend.m2+pend.m3)/(2*pend.kx1);
                            pend.bend2 = pend.g*(pend.m2+pend.m3)/(2*pend.kx2);
                              
                            
                            pend.dm = (1+0)/1000 - pend.flexn;
                            pend.dn = (1+0)/1000 - pend.flex1 + pend.bend1;
                            pend.d0 = (1+0)/1000 - pend.flex1;
                            pend.d1 = (1+0)/1000 - pend.flex2 + pend.bend2;
                            pend.d2 = 10/1000 - pend.flex2;
                            pend.d3 = (1-0)/1000; % T1000518 
                            pend.d4 = (1-0)/1000; % T1000518 
                           
%                             pend.dn = 0.001 - pend.flex1 + pend.bend1;%
%                             pend.dn = 0.003263156834839;% + 0.1311/1000; % Model Fit: gradient_descent_fit_y_pitch_roll_UI_locked; BNS 5/9/09
%                             pend.d0 = 0.001 - pend.flex1; % BNS 6/18/08
%                             pend.d0 = 0.000939441915619; % Model Fit: gradient_descent_fit_y_pitch_roll_PUM_locked; BNS 5/9/09
%                             pend.d1 = 0.001 - pend.flex2 + pend.bend2; , BNS 5/9/09
%                             pend.d1 = -0.2668729892457283/1000;% + 0.1211/1000; % Model Fit: gradient_descent_fit_y_pitch_roll_PUM_locked; BNS 5/9/09                                                       
%                             pend.d4 = 1.0000*10^-03 - pend.flex3; % BNS 6/18/08  
%                             pend.d4 = 0.002 - pend.flex3; % BNS 5/9/09, From 'Proposed Steel Prism Position (from D060039)'
              
                    
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            %    Test Mass Wire Stiffnesses     %
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            % added for consitency with monolithic model,
                            % where the fiber stiffness is adjustable. BNS
                            % 20 March 2011
                            kw3	= pend.Y3*pi*pend.r3^2/pend.l3*pend.nw3/2;
                            
                                                     

% Useful Density values (kg/m^3), BNS 5/9/09
% Density of 304 stainless steel  8000    http://www.azom.com/Details.asp?ArticleID=965
% Density of 5083 aluminum        2650    http://www.azom.com/Details.asp?ArticleID=2804
% Density of fused silica         2201    http://www.sciner.com/Opticsland/FS.htm
% Density of air                  1.225   at 20 C, sea level, 20% humididty; http://en.wikipedia.org/wiki/Atmospheric_pressure
                            
% Output of quad_ref.m
%	longpitch1: [0.3976 0.4434 0.9894 1.1708]
%	longpitch2: [1.9828 2.8193 3.2322 3.4046]
%	       yaw: [0.6846 1.4288 2.5359 3.1652]
%	transroll1: [0.4640 0.8079 1.0443 2.0957]
%	transroll2: [2.7519 3.3251 5.0459 25.1226]
%	  vertical: [0.5814 2.3255 3.7326 17.3318]


% Stage2 output of Mathematica for comparison
%	N	f	type
%	1	0.3975644352339722	pitch3	pitch2
%	2	0.4433775561541977	x3	x2
%	3	0.4639809669357563	y3
%	4	0.5814249875070461	z3	z2
%	5	0.6846092071853637	yaw3	yaw1
%	6	0.8079415294919186	roll1	roll2	roll3
%	7	0.9894144499624277	x2	x3
%	8	1.044307242552718	y2	y3
%	9	1.170812753787754	pitch0	pitch1
%	10	1.428841235069959	yaw3	yaw1
%	11	1.982827019933515	x0	x2
%	12	2.095734967343347	y0	y1
%	13	2.325469494217848	z0	z1
%	14	2.535886058824618	yaw0	yaw2
%	15	2.751930679786737	roll1	roll0	roll3	roll2
%	16	2.819287432755158	pitch1	pitch0
%	17	3.165249402433968	yaw2	yaw1
%	18	3.232175696418479	pitch2	pitch3
%	19	3.325113056347424	roll1	y1	y0
%	20	3.404592789682972	x1	x0
%	21	3.732607589631745	z1	z0
%	22	5.045900440685648	roll0	y0
%	23	17.33177412817714	z2	z3
%	24	25.12255788363513	roll2	roll3