function estimator = generate_freq_longitudinal_estimator(quad_mod_longitudinal,sensor_cost_filter)
% LQR optimal estimator design for modal damping a quad's 4 longitudinal modes.
% The LQR optimization includes a frequency domain weight, sensor_cost_filter, on the noisy
% sensor signal and scalar weights for the reconstruction of the 4 modal
% states (in the modal basis, not the euler basis).
% The frequency domain weight on the sensor allows one to quatify how much 
% the sensor should be trusted at each frequency. More gain means less
% trust.
%
% The frequency domain cost function, sensor_cost_filter, must have equal numbers of poles and
% zeros to ensure that the LQR R matrix fulfills its positive definte 
% requirement (R > 0).
%
% quad_mod_longitudinal is a struct containing the reduced order quad
% pendulum model. It is reduced order meaning it only contains longitudinal
% dynamics; no pitch, roll, etc.
%
% sensor_cost_filter must be a zpk format. In principal
% it could be any filter format, but the function is setup assuming the
% filter comes into the function as a zpk.
%
% The function has 5 sections seperated by cells:
% 1) Load the reduced order longitudinal quad model
% 2) Convert the cost filter into state space form
% 3) Establish the weights on estimating each mode. These are scaler
%    values.
% 4) Run the LQR otpimization to generate optimal estimator feedback
% 5) Build the estimator state space model.
%
% Refernces:
% 1) T1300301
% 2) Narendra Gupta. Frequency-Shaped Cost Functionals: Extension of Linear-Quadratic-
% Guassian Design Methods. Journal of Guidance, Control, and Dynamics, 3(6):529{535,
% 1980.
%
% Brett Shapiro
% 17 May 2011
%
% General cleanup
% Brett Shapiro
% 19 March 2013


%% 1) Load the reduced order longitudinal quad model

quad_long_mod = quad_mod_longitudinal.plant; % all dynamics other than longitudinal are removed

num_sensor  = quad_mod_longitudinal.num_sensor; % number of sensor signals
num_seismic = quad_mod_longitudinal.num_seismic; % number of seismic inputs to model
num_dof     = quad_mod_longitudinal.num_dof; % number of pendulum degrees of freedom


%% 2) Frequency domain cost function on noisy sensor. High gain regions are where the sensor is not trusted.

[zu1 pu1 ku1] = zpkdata(sensor_cost_filter);
    zu1 = zu1{1};  % convert cell to vector
    pu1 = pu1{1};  % convert cell to vector
nu1 = length(pu1); % dimension of the weighting filter

% convert the filter to state space form to interface with the LQR optimization
[Afwu1 Bfwu1 Cfwu1 Dfwu1] = zp2ss(zu1,pu1,ku1);


%% 3) Modal scalar weights. 
% Determine the cost of estimation errors on each mode. The higher the value, the better the estimation, at the expense of poorer noise performance.
% The weights here are scalar values, they are not in the frequency domain.

% preliminary data
np = 2*num_dof; % number of pendulum states or poles (twice the number of degrees of freedom)
[phi modes] = ModalDecomp2(quad_long_mod); % modal decomposition; phi is modal vectors, modes is the mode frequencies
M = quad_long_mod.b(num_dof+1:end,1+num_seismic:end)\eye(num_dof); % these are the masses of the pendulum stages

% converting the pendulum into modal coordiantes
Mm = diag(diag(phi'*M*phi)); % these are the modal masses (effective mass in modal coordinates)
Am = quad_long_mod.a; % initialization of modal state space A matrix
Am(num_dof+1:num_dof*2,1:num_dof)=phi\Am(num_dof+1:num_dof*2,1:num_dof)*phi; % calculation of modal state space A matrix
Bm = quad_long_mod.b(:,num_seismic+1:end); % initialization of modal state space B matrix
Bm(num_dof+1:2*num_dof,:) = phi\Bm(num_dof+1:2*num_dof,:); % calculation of modal state space B matrix
Ckn = [eye(num_sensor) zeros(num_sensor,2*num_dof-num_sensor)]; % state space C matrix representing the available measurements (in the euler basis)
Cknm = [Ckn(:,1:num_dof)*phi zeros(num_sensor,num_dof)]; % calculation of the modal state space C matrix with

% generating modal weights (subset of the lqr 'Q' matrix)
Qm = zeros(2*num_dof);
for ii=1:num_dof
    Qm(ii,ii) = 0; % Zero weight on the modal position because we want to apply velocity damping. If we want to weight displacement, chose 1/(Mm(ii,ii)*modes(ii)*2*pi), which is the modal impulse response amplitude.
    Qm(ii+num_dof,ii+num_dof) = 1/Mm(ii,ii); %  modal velocity impulse reponse amplitude
end
Qm = Qm.^2; % The cost function is specified in terms of squared states. This opperation makes Q consistent with the cost function. % x -> R = 7.1e-7, z -> R = 4.975e-6; when velocity term is zero Rx = 3.578e-7, Rz = 2.466e-6


%% 4) LQR optimization

% 'Superplant' with modal plant and weighting function
Ass = [      Am'        zeros(np,nu1)
       zeros(nu1,np)      Afwu1     ];
   
Bss = [   Cknm'
         Bfwu1 ];

% Q matrix to input to lqr function
Q = [     Qm       zeros(np,nu1)
     zeros(nu1,np) Cfwu1'*Cfwu1];%diag([q0 Cfwu1'*Cfwu1]);
 
% R matrix to input to lqr function
R = Dfwu1'*Dfwu1;

% N matrix to input to lqr function
N = [zeros(np,1)
     Cfwu1'*Dfwu1];
 
% Optimal feedback matrix determined by lqr function. Note that this
% feedback matrix requires feedback of all states, inclduing the nu1 states
% from the weighting filter
Lss = lqr(Ass,Bss,Q,R,N)';
Lm = Lss(1:np);     % the feedback matrix components involving the pendulum states
Lw = Lss(np+1:end); % the feedback matrix components involving the weighting filter states

%% 5) Building the estimator state space equations
estimator.A = [ (Am-Lm*Cknm)     -Lm*Bfwu1'
                  -Lw*Cknm   (Afwu1'-Lw*Bfwu1')];
estimator.B = [     Bm          Lm
               zeros(nu1,num_dof)  Lw  ];
estimator.C = [zeros(num_dof) eye(num_dof) zeros(num_dof,nu1)];
estimator.D = zeros(size(estimator.C,1),size(estimator.B,2));
estimator.L = Lss; % this field not used, just an FYI of the optimal lqr feedback matrix
estimator.ss = ss(estimator.A,estimator.B,estimator.C,estimator.D); % estimator state space